
SIPURandom class extend [
    nextByte [
        <category: 'random'>
        ^(RAND rand: 1) first.
    ]

    nextInt [
        <category: 'random'>
        ^(RAND rand: 4) asInteger
    ]
]

SIPUserAgentBase class extend [
    generateCSeq [
        <category: 'helper'>
        "For pharo just use the random we have"
        ^(RAND rand: 4) asInteger abs
    ]
]

SIPQuotedStringParser extend [
    skipWhitespace: aStream [
        [aStream atEnd] whileFalse:  [
            | c |
            c := aStream uncheckedPeek.
            c = Character tab ifTrue: [aStream next].
            c = Character space ifTrue: [aStream next].
            ^self
        ]
    ]


    parseOn: aContext [
        | memento startPosition result |
        startPosition := aContext position.
        memento := aContext remember.

        "Skip whitespace"
        self skipWhitespace: aContext.

        "Check for the opening space"
        aContext atEnd ifTrue: [
            result := PPFailure message: 'No space for opening quote' context: aContext.
            aContext restore: memento.
            ^result].

        aContext uncheckedPeek = $" ifFalse: [
            result := PPFailure message: 'No opening quote' context: aContext.
            aContext restore: memento.
            ^result].

        aContext skip: 1.
        result := self parseToClosingQuote: aContext.
        result isPetitFailure ifTrue: [
            aContext restore: memento].
        ^result
    ]

    parseToClosingQuote: aStream [
        | text inQuote finish parsed |
        text := WriteStream on: String new.
        inQuote := false.
        parsed := false.
        finish := aStream atEnd.
        [finish] whileFalse: ["Did we have an escape?"

            inQuote
                ifTrue: [
                    "TODO: Check if that is a valid sequence"

                    text nextPut: aStream next.
                    inQuote := false.
                    finish := aStream atEnd]
                ifFalse: [
                    | c |
                    c := aStream uncheckedPeek.
                    c = $"
                        ifTrue: [
                            aStream skip: 1.
                            parsed := true.
                            ^text contents]
                        ifFalse: [
                            c = $\ ifTrue: [inQuote := true].
                    text nextPut: c.
                    aStream skip: 1.
                    finish := aStream atEnd]]].
        ^PPFailure message: 'Expected closing quote' context: aStream
    ]
]

Base64LikeConverter class extend [

    mimeEncode: aStream [
	"Return a ReadWriteStream of characters.  The data of aStream is encoded as 65 innocuous characters.  (See class comment). 3 bytes in aStream goes to 4 bytes in output."

	| me |
	me := self new dataStream: aStream.
	me 
	    mimeStream: (ReadWriteStream on: (String new: (aStream size + 20) * 4 // 3)).
	me mimeEncode.
	me mimeStream position: 0.
	^me mimeStream
    ]
]

SIPDigest class extend [
    ha1: aName realm: aRealm password: aPassword [
        ^(MD5 hashMessage: (String streamContents: [:str |
            str
                nextPutAll: aName;
                nextPutAll: ':';
                nextPutAll: aRealm;
                nextPutAll: ':';
                nextPutAll: aPassword])) hex.
    ]

    ha2: anOperation uri: aSipUrlString [
        ^(MD5 hashMessage: (String streamContents: [:str |
            str
                nextPutAll: anOperation;
                nextPutAll: ':';
                nextPutAll: aSipUrlString])) hex.
    ]

    authUser: aName password: aPassword realm: aRealm nonce: aNonce operation: anOperation url: aSipUrlString [
        | ha1 ha2 resp md5 |
        ha1 := self ha1: aName realm: aRealm password: aPassword.
        ha2 := self ha2: anOperation uri: aSipUrlString.

        ^(MD5 hashMessage: (String streamContents: [:str |
            str
                nextPutAll: ha1;
                nextPutAll: ':';
                nextPutAll: aNonce;
                nextPutAll: ':';
                nextPutAll: ha2])) hex
    ]

    authUser: aName password: aPassword realm: aRealm nonce: aNonce operation: anOperation url: aSipUrlString qop: aQop clientNonce: aCnonce nonceCount: aNc [
        | ha1 ha2 resp md5 |
        ha1 := self ha1: aName realm: aRealm password: aPassword.
        ha2 := self ha2: anOperation uri: aSipUrlString.

        ^(MD5 hashMessage: (String streamContents: [:str |
            str
                nextPutAll: ha1;
                nextPutAll: ':';
                nextPutAll: aNonce;
                nextPutAll: ':';
                nextPutAll: aNc;
                nextPutAll: ':';
                nextPutAll: aCnonce;
                nextPutAll: ':';
                nextPutAll: aQop;
                nextPutAll: ':';
                nextPutAll: ha2])) hex
    ]
]
